# coding: utf-8

from educommon.utils.fonts import ARIAL, get_font, split_text

# Высота и ширина символа "0" шрифта по умолчанию в xlwt.
DEFAULT_CHAR_SIZE = 256


def adjust_row_height(section, row_idx, col_idx, text, font):
    u"""Увеличивает высоту строки, если необходимо.

    Высота строки устанавливается такая, чтобы текст `text`
    поместился в ячейку с индексами `row_idx`, `col_idx`.

    Можно применять для нескольких ячеек одной строки. Тогда строке будет
    установлена максимальная высота.

    Не работает с объединенными ячейками!

    :param section: Секция, в которой находится строка.
    :type section: :class:`simple_report.xls.section.Section`
    :param int row_idx: Индекс строки в таблице excel.
    :param int col_idx: Индекс столбца в таблице excel.
    :param unicode text: Строка, которая будет записана в ячейку.
    :param int font_size: Размер шрифта.
    :rtype: :class:`PIL.ImageFont`
    """
    def count_row_height(text, column_width):
        u"""Вычисляет высоту строки отчета в зависимости от текста."""

        # Т.к. высота и ширина ячейки измеряется в единицах относительно
        # дефолтного шрифта, берем его параметры.
        normal_font = get_font(ARIAL, 10)
        width_in_chars = column_width / DEFAULT_CHAR_SIZE
        str_width_px, str_height_px = normal_font.getsize('0' * width_in_chars)

        # Определяем коэффициент масштабирования высоты шрифта к дефолтному.
        _, choosen_font_height_px = font.getsize('0')
        height_scale = (choosen_font_height_px * 1.0) / str_height_px

        rows_count = len(
            split_text(text, font, str_width_px)
        )

        height = (
            int(rows_count * height_scale * DEFAULT_CHAR_SIZE) +
            DEFAULT_CHAR_SIZE / 4
        )

        return height

    row = section.writer.wtsheet.row(row_idx)
    col = section.writer.wtsheet.col(col_idx)

    row.height = max(
        row.height,
        count_row_height(text, col.width)
    )


def adjust_row_height_arial(section, row_idx, col_idx, text, font_size=10):
    u"""Устанавливает высоту строки автоматически.

    Высота строки устанавливается такая, чтобы текст text
    поместился в ячейку с индексами row_idx, col_idx.

    Расчет ведется для шрифта Arial, и соответствует отображению ячеек в
    MS Excel.

    Не работает с объединенными ячейками!

    :param section: Секция, в которой находится строка
    :type section: :class:`simple_report.xls.section.Section`
    :param int row_idx: Индекс строки в таблице excel
    :param int cell_idx: Индекс столбца в таблице excel
    :param unicode text: Строка, которая будет записана в ячейку
    :param int font_size: Размер шрифта в пунктах
    """
    font = get_font(ARIAL, font_size)
    adjust_row_height(section, row_idx, col_idx, text, font)
